#include <joyos.h>
#include <lib/pid.h>

#define RightMotor 0
#define LeftMotor 3

#define RightEncoder 26
#define LeftEncoder 25

#define TargetVelocity 100
#define MaxVelocity 250

uint16_t PIDDelay;

float EncoderDifferential(); // Defined to be Right - Left

void ForwardApplyMV(float);
void LeftApplyMV(float);
void RightApplyMV(float);

float BoundedVelocity(float);
void SetRightVelocity(float);
void SetLeftVelocity(float);
void ResetEncoders();
void CoastMotors();
void CoastMotor(uint8_t);

void ResetEncoders()
{
	encoder_reset(RightEncoder);
	encoder_reset(LeftEncoder);
}

float EncoderDifferential()
{
	return (float)encoder_read(RightEncoder)-(float)encoder_read(LeftEncoder);
}

float BoundedVelocity(float ProposedVelocity)
{
	if (ProposedVelocity < 0)
	{
		return 0;
	}
	else if (ProposedVelocity > MaxVelocity)
	{
		return MaxVelocity;
	}
	else
	{
		return ProposedVelocity;
	}
}

void SetRightVelocity(float vel)
{
	motor_set_vel(RightMotor, (int16_t)vel);
}

void SetLeftVelocity(float vel)
{
	motor_set_vel(LeftMotor, (int16_t)vel);
}

void ForwardApplyMV(float MV)
{
	SetRightVelocity(BoundedVelocity(TargetVelocity+MV/2));
	SetLeftVelocity(BoundedVelocity(TargetVelocity-MV/2));	
}

void MotorCoast(uint8_t Motor)
{
	motor_set_vel(Motor, 0);
}

void CoastMotors()
{
	MotorCoast(RightMotor);
	MotorCoast(LeftMotor);
}

// usetup is called during the calibration period. It must return before the
// period ends.
int usetup (void) {
	set_auto_halt(0);
	return 0;
}

int
umain(void)
{
	// Run for 2 minutes
	uint32_t StopTime = get_time()+120000;

	ResetEncoders();

	uint16_t kp, ki, kd;

	do
	{
		printf("\nT: %d", PIDDelay=frob_read_range(1,1000));
		pause(50);
	} while (!go_press());

	go_click();

	do
	{
		printf("\nKp: %d", kp=frob_read_range(0,200));
		pause(50);
	} while (!go_press());

	go_click();

	do
	{
		printf("\nKi: %d", ki=frob_read_range(0,100));
		pause(50);
	} while (!go_press());

	go_click();

	do
	{
		printf("\nKd: %d", kd=frob_read_range(0,25));
		pause(50);
	} while (!go_press());

	printf("\nPress GO...");

	go_click();

	printf("\n");

	struct pid_controller ForwardPID;

	init_pid(	&ForwardPID,
			(float)kp,
			(float)ki,
			(float)kd,
			&EncoderDifferential,
			&ForwardApplyMV);
	ForwardPID.goal = 0.0;
	ForwardPID.enabled = 1;

	
	while (get_time() < StopTime)
	{
		update_pid(&ForwardPID);

		pause(PIDDelay);
	}

	CoastMotors();

	return 0;
}

